USE [igops]
GO

DROP VIEW IF EXISTS escalation_reviewer_v
GO

DROP VIEW IF EXISTS review_item_action_excp_v
GO

CREATE VIEW review_item_action_excp_v AS
SELECT 
  action.review_instance_id,
  COUNT(DISTINCT action.review_item_id) AS count
FROM review_item_action action
  JOIN review_instance ON action.review_instance_id = review_instance.id AND review_instance.status NOT IN ('PREVIEW', 'PREVIEW_CANCELED', 'PREVIEW_STARTING')
WHERE action.caused_by_exception = '1' OR action.reviewer_type = 'EXCEPTION_REVIEWER' OR action.reviewer_type = 'RO'
GROUP BY action.review_instance_id;
GO

CREATE VIEW escalation_reviewer_v AS
  SELECT 
    item.id,
    instance.id AS review_instance_id,
    defn.name AS review_name,
    defn.description AS review_desc,
    instance.status AS review_status,
    defn.review_type,
    owners.review_owners,
    CASE
      WHEN inst_multi_esc_reviewers.display_name IS NOT NULL THEN inst_multi_esc_reviewers.display_name
      ELSE inst_single_esc_reviewer.display_name
    END AS inst_esc_reviewers,
  CASE
      WHEN action.shared_queue = 1 THEN item_multi_esc_reviewers.display_name
      ELSE item_single_esc_reviewer.display_name
    END AS item_esc_reviewers,
    action.reviewer_comment,
    action.caused_by_exception_type,
    CASE
      WHEN item.item_type = 'PERMISSION' THEN perm.name
      WHEN item.item_type = 'ACCOUNT'THEN acct.name
      WHEN item.item_type = 'ROLE' THEN role.name
      WHEN item.item_type = 'BUSINESS_ROLE' THEN brole.name
    WHEN (item.item_type = 'USER' OR item.item_type = 'DIRECT_REPORT') THEN suser.display_name
      ELSE NULL
    END AS review_item,
    action.assigned
  FROM review_item item
    JOIN review_instance instance ON instance.id = item.review_instance_id AND instance.status NOT IN ('PREVIEW', 'PREVIEW_CANCELED', 'PREVIEW_STARTING')
    JOIN (SELECT
        review_instance.id inst_id,
        defn.id defn_id,
        defn.review_type,
        CASE WHEN staged_defn.name IS NULL THEN defn.name ELSE staged_defn.name END AS name,
        CASE WHEN staged_defn.description IS NULL THEN defn.description ELSE staged_defn.description END description
      FROM review_definition defn
        JOIN review_instance ON defn.id = review_instance.review_definition
        LEFT JOIN review_definition staged_defn ON staged_defn.id = review_instance.staged_review_definition_id) defn ON defn.inst_id = instance.id
    JOIN review_item_action action ON item.id = action.review_item_id AND action.caused_by_exception = 1 AND (action.review_item_status != 'OVERRIDDEN' AND action.action != 'REASSIGNED')
    LEFT JOIN saccount acct ON acct.unique_account_id = item.unique_account_id AND acct.effective = 1
    LEFT JOIN spermission perm ON perm.unique_permission_id = item.unique_permission_id AND perm.effective = 1
    LEFT JOIN role_policy role ON role.id = item.role_id
    LEFT JOIN business_role brole ON brole.id = item.business_role_id
    JOIN (SELECT 
        ri.id rid,
        STUFF((SELECT ' | ' + s.display_name FROM review_instance ri1
            JOIN review_definition defn_1 ON ri1.review_definition = defn_1.id
            LEFT JOIN reviewer_instance rev_owner ON ri1.id = rev_owner.review_instance_id AND rev_owner.type = 'RO'
            LEFT JOIN reviewer_user_instance rui ON rui.reviewer_instance_id = rev_owner.id
            LEFT JOIN suser s ON rui.unique_user_id = s.unique_user_id AND s.effective = 1 
          WHERE ri1.ID = ri.id FOR XML PATH('')),1,3,'') review_owners
      FROM review_instance ri
        JOIN review_definition defn_1 ON ri.review_definition = defn_1.id
        LEFT JOIN reviewer_instance rev_owner ON ri.id = rev_owner.review_instance_id AND rev_owner.type = 'RO'
        LEFT JOIN reviewer_user_instance rui ON rui.reviewer_instance_id = rev_owner.id
        LEFT JOIN suser s ON rui.unique_user_id = s.unique_user_id AND s.effective = 1
      GROUP BY defn_1.name, ri.id) owners ON owners.rid = instance.id
    LEFT JOIN (SELECT
        rev_inst.id AS instance_id,
        suser.display_name
      FROM review_instance rev_inst
        JOIN reviewer_instance ri ON rev_inst.id = ri.review_instance_id AND ri.type = 'EO' AND ri.shared_queue = 0
        JOIN suser ON ri.single_reviewer_unique_id = suser.unique_user_id AND suser.effective = 1) inst_single_esc_reviewer ON inst_single_esc_reviewer.instance_id = instance.id
    LEFT JOIN (SELECT 
        rev_inst.id AS instance_id,
        STUFF((SELECT ' | '  + suser.display_name FROM review_instance rev_inst_1
            JOIN reviewer_instance ri ON rev_inst_1.id = ri.review_instance_id AND ri.type = 'EO'
            JOIN reviewer_set_instance rsi ON ri.id = rsi.reviewer_instance_id
            JOIN suser ON rsi.unique_user_id = suser.unique_user_id AND suser.effective = 1
          WHERE rev_inst_1.id = rev_inst.id 
          ORDER BY suser.display_name FOR XML PATH('')), 1, 3, '') display_name
      FROM review_instance rev_inst
        JOIN reviewer_instance ri ON rev_inst.id = ri.review_instance_id AND ri.type = 'EO'
        JOIN reviewer_set_instance rsi ON ri.id = rsi.reviewer_instance_id
        JOIN suser ON rsi.unique_user_id = suser.unique_user_id AND suser.effective = 1
      GROUP BY rev_inst.id) inst_multi_esc_reviewers ON inst_multi_esc_reviewers.instance_id = instance.id
    LEFT JOIN (SELECT 
        action.review_item_id AS item_id,
        suser.display_name
      FROM review_item_action action
        JOIN reviewer_instance ri ON action.reviewer_instance_id = ri.id AND ri.shared_queue = 0
        JOIN suser ON ri.single_reviewer_unique_id = suser.unique_user_id AND suser.effective = 1
      WHERE action.caused_by_exception = 1 AND (action.review_item_status != 'OVERRIDDEN' AND action.action != 'REASSIGNED')) item_single_esc_reviewer ON item_single_esc_reviewer.item_id = item.id
    LEFT JOIN (SELECT
        action_1.review_item_id AS item_id,
        STUFF((SELECT ' | '  + suser_1.display_name FROM review_item_action action_2
            JOIN reviewer_instance ri_1 ON action_2.reviewer_instance_id = ri_1.id AND ri_1.shared_queue = 1
            JOIN reviewer_set_instance rsi_1 ON ri_1.id = rsi_1.reviewer_instance_id
            JOIN suser suser_1 ON rsi_1.unique_user_id = suser_1.unique_user_id AND suser_1.effective = 1
          WHERE action_1.review_item_id = action_2.review_item_id AND action_1.caused_by_exception = 1 
          ORDER BY suser_1.display_name FOR XML PATH('')), 1, 3, '') display_name
      FROM review_item_action action_1
        JOIN reviewer_instance ri ON action_1.reviewer_instance_id = ri.id AND ri.shared_queue = 1
        JOIN reviewer_set_instance rsi ON ri.id = rsi.reviewer_instance_id
        JOIN suser ON rsi.unique_user_id = suser.unique_user_id AND suser.effective = 1
      WHERE action_1.caused_by_exception = 1
      GROUP BY action_1.review_item_id, action_1.caused_by_exception) item_multi_esc_reviewers ON item_multi_esc_reviewers.item_id = item.id
    LEFT JOIN suser ON suser.unique_user_id = item.unique_user_id AND suser.effective = 1
GO


GRANT SELECT ON review_item_action_excp_v TO ig_report_role
GO

GRANT SELECT ON escalation_reviewer_v TO ig_report_role
GO